@@ -1,6 +1,4 @@
 from __future__ import absolute_import
-from builtins import next
-from builtins import object
 import os
 import posixpath
 import re
@@ -12,6 +10,9 @@ import mimetypes
 import base64
 import uuid
 
+import six
+from builtins import next, object
+
 from .utils.encoding import encode, decode
 
 logger = logging.getLogger(__name__)
@@ -28,6 +29,7 @@ tags:
 - example
 created_at: 2016-06-29
 updated_at: 2016-06-30
+thumbnail: 2
 tldr: |
     You can write any markdown you want here (the '|' character makes this an escaped section)
 
@@ -61,7 +63,7 @@ def setup_yaml():
     def dict_constructor(loader, node):
         return collections.OrderedDict(loader.construct_pairs(node))
 
-    yaml.add_representer(collections.OrderedDict, dict_representer)
+    yaml.SafeDumper.add_representer(collections.OrderedDict, dict_representer)
     yaml.add_constructor(_mapping_tag, dict_constructor)
 
 
@@ -134,15 +136,15 @@ class ReferenceCache(object):
         for key in cache:
             if isinstance(cache[key], dict):
                 for path in self.dir(parent=posixpath.join(parent or '', key), cache=cache[key]):
-                    yield path
+                    yield posixpath.join(key, path)
             else:
-                yield posixpath.join(parent or '', key)
+                yield key
 
 
 class KnowledgePost(object):
     '''
     A "Knowledge Post" is a (virtual) folder in which there is a 'knowledge.md' file,
-    and potentially an 'images' and/or 'orig_src' folder. It is "virtual" in the sense
+    and potentially an 'images' and/or 'src' folder. It is "virtual" in the sense
     that `KnowledgePost` objects store everything in memory, and use `KnowledgeRepository`
     object instances to interface with "physical" realisations of them. For example,
     in a GitKnowledgeRepository, the physical realisation is an actual folder; whereas
@@ -198,7 +200,7 @@ class KnowledgePost(object):
 
     def _dir(self, parent=None):
         for ref in self.__cache.dir(parent=parent):
-            if self.__cache[ref] is not None:
+            if self.__cache[posixpath.join(parent or '', ref)] is not None:
                 yield ref
         if self.repository is not None:
             for ref in self.repository._kp_dir(self.path, parent=parent, revision=self.revision):
@@ -212,11 +214,11 @@ class KnowledgePost(object):
             md = ''
         else:
             md = decode(self._read_ref('knowledge.md'))
-            mtch = re.match('^---\n[\s\S]+?---\n', md)
+            mtch = re.match('^---\n[\\s\\S]+?---\n', md)
             if not mtch:
                 raise ValueError("YAML header is missing. Please ensure that the top of your post has a header of the following form:\n" + SAMPLE_HEADER)
             if not headers:
-                md = re.sub('^---\n[\s\S]+?---\n', '', md, count=1)
+                md = re.sub('^---\n[\\s\\S]+?---\n', '', md, count=1)
             if not body:
                 md = mtch.group(0)
         if images:
@@ -228,7 +230,7 @@ class KnowledgePost(object):
         return ['images/{}'.format(image_name) for image_name in self._dir(parent='images')]
 
     def read_image(self, name):
-        return self._read_ref('image/' + name)
+        return self._read_ref('images/' + name)
 
     def read_images(self):
         image_data = {}
@@ -236,15 +238,28 @@ class KnowledgePost(object):
             image_data[image_path] = self._read_ref(image_path)
         return image_data
 
+    @property
+    def src_paths(self):
+        srcs = ['src/{}'.format(src_name) for src_name in self._dir(parent='src')]
+        legacy_srcs = ['orig_src/{}'.format(src_name) for src_name in self._dir(parent='orig_src')]  # TODO: deprecate
+        return srcs + legacy_srcs
+
     def read_src(self, ref):
-        return self._read_ref('orig_src/' + ref)
+        # Read src reference, first attempting to read from `src`, then from
+        # the legacy `orig_src`, path.
+        try:
+            return self._read_ref('src/' + ref)
+        except Exception as e:
+            if self._has_ref('orig_src/' + ref):
+                return self._read_ref('orig_src/' + ref)
+            raise e
 
     def write(self, md, headers=None, images={}):
         md = md.strip()
         if headers is not None:
-            md = re.sub('^---\n[\s\S]+?---\n', '', md, count=1)
+            md = re.sub('^---\n[\\s\\S]+?---\n', '', md, count=1)
             md = '---\n' + \
-                yaml.dump(headers, default_flow_style=False) + '---\n' + md
+                yaml.safe_dump(headers, default_flow_style=False) + '---\n' + md
         md += '\n'
 
         self._write_ref('knowledge.md', encode(md))
@@ -260,7 +275,7 @@ class KnowledgePost(object):
             self.write_image(name, data)
 
     def write_src(self, name, data):
-        self._write_ref('orig_src/' + name, encode(data))
+        self._write_ref('src/' + name, encode(data))
 
     def add_srcfile(self, filename, name=None):
         if not name:
@@ -282,43 +297,47 @@ class KnowledgePost(object):
         for key, value in headers.copy().items():
             if isinstance(value, datetime.date):
                 headers[key] = datetime.datetime.combine(value, datetime.time(0))
+            if key == 'tags' and isinstance(value, list):
+                headers[key] = [str(v) if six.PY3 else unicode(v) for v in value]
         return headers
 
+    @headers.setter
+    def headers(self, headers):
+        self.write(self.read(headers=False), headers=headers)
+
     def update_headers(self, **headers):
         h = self.headers
-        h.update(headers)
-        self.write(self.read(), headers=h)
+        for header, value in headers.items():
+            if value is None:
+                if header in h:
+                    h.pop(header)
+            else:
+                h[header] = value
+        self.headers = h
 
     @property
     def thumbnail_uri(self):
-        image_matches = ExtractImages.find_images(self.read())
-        if len(image_matches) == 0:
+        thumbnail = self.headers.get('thumbnail')
+
+        if not thumbnail or not isinstance(thumbnail, str):
             return None
 
-        i = 0
-        image_uri = None
-        while not image_uri:
-            if i == len(image_matches):
+        if ':' not in thumbnail:  # if thumbnail points to a local reference
+            if not self._has_ref(thumbnail):
                 return None
-            image_uri = image_matches[i]['src']
-            i += 1
-
-        if ':' not in image_uri:  # TODO: Check this works
-            if not os.path.exists(image_uri):
-                return None
-            with open(self.feed_image[7:]) as f:
-                data = base64.b64encode(f.read())
-            image_mimetype = mimetypes.guess_type(self.feed_image)[0]
+            data = base64.b64encode(self._read_ref(thumbnail))
+            image_mimetype = mimetypes.guess_type(thumbnail)[0]
             if image_mimetype is not None:
-                return 'data:{};base64,'.format(image_mimetype) + data
+                return 'data:{};base64,'.format(image_mimetype) + data.decode('utf-8')
+            return None
 
-        return image_uri
+        return thumbnail
 
     def is_valid(self):
         if not self._has_ref('knowledge.md'):
             return False
         try:
-            FormatChecks.process(self)
+            FormatChecks().process(self)
         except:
             return False
         return True
@@ -364,6 +383,7 @@ class KnowledgePost(object):
     def to_string(self, format, **opts):
         return KnowledgePostConverter.for_format(self, format).to_string(**opts)
 
+
 from .converter import KnowledgePostConverter  # noqa
 from .postprocessors.format_checks import FormatChecks  # noqa
 from .postprocessors.extract_images import ExtractImages  # noqa
