@@ -1,31 +1,31 @@
 import datetime
 
+import six
+from past.builtins import basestring
+
 from ..postprocessor import KnowledgePostProcessor
 
 REQUIRED_FIELD_TYPES = {
-    'title': str,
+    'title': six.string_types,
     'authors': list,
     'created_at': datetime.datetime,
-    'tldr': str,
+    'tldr': six.string_types,
     'tags': list
 }
 
 OPTIONAL_FIELD_TYPES = {
-    'path': str,
+    'path': six.string_types,
     'updated_at': datetime.datetime,
     'private': bool,   # If true, this post starts out private
-    'allowed_groups': list
+    'allowed_groups': list,
+    'thumbnail': (int, ) + six.string_types
 }
 
 
 class FormatChecks(KnowledgePostProcessor):
-    '''
-    Use this to bootstrap your own KnowledgePostProcessor.
-    '''
     _registry_keys = ['format_checks']
 
-    @classmethod
-    def process(cls, kp):
+    def process(self, kp):
         headers = kp.headers
         for field, typ in REQUIRED_FIELD_TYPES.items():
             assert field in headers, "Required field `{}` missing from headers.".format(
